@@ -51,6 +51,8 @@ namespace BOOM {
     virtual void add_data(const Ptr<DataType> &dp);
     virtual void remove_data(const Ptr<Data> &dp);
 
+    long sample_size() const {return dat_.size();}
+
     std::vector<Ptr<DataType>> &dat() { return dat_; }
     const std::vector<Ptr<DataType>> &dat() const { return dat_; }
 
@@ -61,14 +63,6 @@ namespace BOOM {
     template <class FwdIt>
     void set_data_raw(FwdIt Beg, FwdIt End);
 
-    template <class FwdIt>
-    void add_data_seq(FwdIt Beg, FwdIt End);
-
-    template <class Cont>
-    void add_data_seq(const Cont &c) {
-      this->add_data_seq(c.begin(), c.end());
-    }
-
     virtual void combine_data(const Model &mod, bool just_suf = true);
 
     void signal() {
@@ -135,15 +129,6 @@ namespace BOOM {
     }
   }
 
-  template <class D>
-  template <class FwdIt>
-  void IID_DataPolicy<D>::add_data_seq(FwdIt Beg, FwdIt End) {
-    while (Beg != End) {
-      add_data(*Beg);
-      ++Beg;
-    }
-  }
-
   template <class D>
   void IID_DataPolicy<D>::combine_data(const Model &other, bool) {
     const DataPolicy &d(dynamic_cast<const DataPolicy &>(other));
