@@ -102,22 +102,6 @@ namespace BOOM {
     // Returns this * rhs.transpose().
     Matrix multT(const Matrix &rhs) const;
 
-    // Solve the equation rhs = this * lhs.  It may be the case that some matrix
-    // blocks are non-invertible, in which case calling this function will
-    // report an error.
-    //
-    // The initial reason for this implementing this method is to evaluate the
-    // transition density for state space models when the error distribution is
-    // less than full rank.   In those settings we must be able to evaluate
-    // error_expander->left_inverse( new_state - T * old_state).
-    //
-    // Thus the only blocks that need to implement left_inverse are those used
-    // in an error expander somewhere.
-    virtual Vector left_inverse(const ConstVectorView &rhs) const {
-      report_error("left_inverse is not implemented for this SparseKalmanMatrix.");
-      return Vector(0);
-    }
-
     // Checks that nrow() == i.  Reports an error if it does not.
     void conforms_to_rows(int i) const;
 
@@ -150,6 +134,201 @@ namespace BOOM {
   // Returns lhs * rhs.transpose();
   Matrix multT(const SpdMatrix &lhs, const SparseKalmanMatrix &rhs);
 
+  //======================================================================
+  // The product of several SparseKalmanMatrix objects.  The terms in the
+  // product are stored, and matrix multiplications are evalutated term-by-term.
+  // Terms or their transposes may be stored.
+  class SparseMatrixProduct : public SparseKalmanMatrix {
+   public:
+    SparseMatrixProduct();
+
+    // Args:
+
+    //   term: Add 'term' to the terms in the product.  Terms are added left to
+    //     right in the order they appear.  So if A, B, and C are added in that
+    //     order, the result is A * B * C.
+    //   transpose: If true then the transpose of the matrix is added to the
+    //     product, so products like A * B' * C can be represented.
+    void add_term(const Ptr<SparseKalmanMatrix> &term, bool transpose = false);
+
+    int nrow() const override;
+    int ncol() const override;
+
+    Vector operator*(const Vector &rhs) const override;
+    Vector operator*(const VectorView &rhs) const override;
+    Vector operator*(const ConstVectorView &rhs) const override;
+    Matrix operator*(const Matrix &rhs) const override;
+
+    Vector Tmult(const ConstVectorView &rhs) const override;
+    Matrix Tmult(const Matrix &rhs) const override;
+
+    Matrix dense() const override;
+
+    // this' * N * this, as a SparseMatrixProduct.
+    Ptr<SparseMatrixProduct> sparse_sandwich(const SpdMatrix &N) const;
+
+    // this * N * this' as a SparseMatrixProduct.
+    Ptr<SparseMatrixProduct> sparse_sandwich_transpose(const SpdMatrix &N) const;
+
+    // The diagonal elements of the sparse matrix.  // how to get these??
+    Vector diag() const;
+
+    // The following functions are implemented, but calling them may result in
+    // very large matrices being created.  Call with care.
+    SpdMatrix inner() const override;
+    SpdMatrix inner(const ConstVectorView &weights) const override;
+    Matrix &add_to(Matrix &rhs) const override;
+
+   private:
+    void check_term(const Ptr<SparseKalmanMatrix> &term, bool transpose);
+
+    std::vector<Ptr<SparseKalmanMatrix>> terms_;
+    std::vector<bool> transposed_;
+  };
+
+  // A sum of sparse matrices. The terms of the sum are stored in the object,
+  // and matrix products are evaluated term by term.
+  class SparseMatrixSum : public SparseKalmanMatrix {
+   public:
+    SparseMatrixSum();
+    void add_term(const Ptr<SparseKalmanMatrix> &term, double coefficient = 1.0);
+
+    int nrow() const override;
+    int ncol() const override;
+
+    Vector operator*(const Vector &rhs) const override;
+    Vector operator*(const VectorView &rhs) const override;
+    Vector operator*(const ConstVectorView &rhs) const override;
+    Matrix operator*(const Matrix &rhs) const override;
+
+    Vector Tmult(const ConstVectorView &rhs) const override;
+    Matrix Tmult(const Matrix &rhs) const override;
+
+    Matrix &add_to(Matrix &rhs) const override;
+
+    // Calling 'inner' on a SparseMatrixSum can result in very large matrices
+    // being created.
+    SpdMatrix inner() const override;
+    SpdMatrix inner(const ConstVectorView &weights) const override;
+
+   private:
+    std::vector<Ptr<SparseKalmanMatrix>> terms_;
+    Vector coefficients_;
+  };
+
+  //===========================================================================
+  // Let M = A + UCV.  The Woodbury identity states that Minv = Ainv - Ainv U
+  // (Cinv + V Ainv U) V Ainv.  Note that to use the Woodbury identity both A
+  // and C must be invertible.
+  class WoodburyInverse : public SparseKalmanMatrix {
+   public:
+
+    // This constructor assumes V is U->transpose().
+    WoodburyInverse(const Ptr<SparseKalmanMatrix> &Ainv,
+                    const Ptr<SparseKalmanMatrix> &U,
+                    const SpdMatrix &C)
+        : Ainv_(Ainv),
+          U_(U),
+          C_(C)
+    {}
+
+   private:
+    Ptr<SparseKalmanMatrix> Ainv_;
+    Ptr<SparseKalmanMatrix> U_;
+    SpdMatrix C_;
+  };
+  //===========================================================================
+
+  // The binomial inverse theorem is a generalization of the Woodbury
+  // identity. The generalization works around an assumption in the Woodbury
+  // identity that the middle matrix in the update term is invertible.  That
+  // need not be the case.  Let M = A + UBV.  The binomial inverse theorem says
+  // Minv = Ainv - Ainv * U * (I + B V Ainv U).inv * B * V * Ainv.
+  //
+  // This class assumes V is U.transpose, and that B is symmetric.  In the
+  // general case these assumptions need not be true.
+  class SparseBinomialInverse : public SparseKalmanMatrix {
+   public:
+
+    // Build a SparseBinomialInverse from raw inputs.
+    //
+    // Args:
+    //   Ainv: The matrix inverse of the "A" matrix in the formula.  This matrix
+    //     is typically highly structured and easy to invert (like a diagonal
+    //     matrix).
+    //   U: The leading term in the product of three matrices comprising the
+    //     update term.
+    //   B: The middle matrix in the update term.  Note that this is a dense
+    //     matrix, while the others are sparse.
+    //   Ainv_logdet: The log determinant of Ainv.  This can be omitted if
+    //     object's "logdet" method will not be called.
+    SparseBinomialInverse(const Ptr<SparseKalmanMatrix> &Ainv,
+                          const Ptr<SparseKalmanMatrix> &U,
+                          const SpdMatrix &B,
+                          double Ainv_logdet = negative_infinity());
+
+    // Reconstitute a SparseBinomialInverse from a previously constructed
+    // matrix.
+    //
+    // Args:
+    //   Ainv: The matrix inverse of the "A" matrix in the formula.  This matrix
+    //     is typically highly structured and easy to invert (like a diagonal
+    //     matrix).
+    //   U: The leading term in the product of three matrices comprising the
+    //     update term.
+    //   B: The middle matrix in the update term.  Note that this is a dense
+    //     matrix, while the others are sparse.
+    //   inner: The "inner matrix" from a previously built
+    //     SparseBinomialInverse.
+    //   logdet: The log determinant from a previously built
+    //     SparseBinomialInverse.
+    SparseBinomialInverse(const Ptr<SparseKalmanMatrix> &Ainv,
+                          const Ptr<SparseKalmanMatrix> &U,
+                          const SpdMatrix &B,
+                          const Matrix &inner,
+                          double logdet);
+
+    // The number of rows and columns in the matrix.
+    int nrow() const override {return Ainv_->nrow();}
+    int ncol() const override {return Ainv_->ncol();}
+
+    Vector operator*(const Vector &rhs) const override;
+    Vector operator*(const VectorView &rhs) const override;
+    Vector operator*(const ConstVectorView &rhs) const override;
+    Matrix operator*(const Matrix &rhs) const override;
+
+    // Implementation assumes the matrix is symmetric.
+    Vector Tmult(const ConstVectorView &rhs) const override;
+    Matrix Tmult(const Matrix &rhs) const override;
+
+    Matrix &add_to(Matrix &rhs) const override;
+
+    // Calling 'inner' on a SparseMatrixSum can result in very large matrices
+    // being created.
+    SpdMatrix inner() const override;
+    SpdMatrix inner(const ConstVectorView &weights) const override;
+
+    Matrix dense() const override;
+
+    // If Ainv_logdet was passed to the constructor, then this returns the log
+    // determinant of the stored matrix.  Otherwise it returns
+    // negative_infinity.
+    double logdet() const;
+
+    const Matrix & inner_matrix() const {return inner_matrix_;}
+
+   private:
+    Ptr<SparseKalmanMatrix> Ainv_;
+    Ptr<SparseKalmanMatrix> U_;
+    SpdMatrix B_;
+
+    // The inner matrix is (I + B * U' * Ainv * U).inv
+    // In the usual case A = H, B = P, U = Z, so
+    // inner = (I + P * Z' Hinv Z).inv
+    Matrix inner_matrix_;
+    double logdet_;
+  };
+
   //======================================================================
   // The SparseMatrixBlock classes are SparseKalmanMatrices that can be used as
   // elements in a BlockDiagonalMatrix.
@@ -189,11 +368,9 @@ namespace BOOM {
     virtual void add_to_block(SubMatrix block) const = 0;
     Matrix &add_to(Matrix &P) const override;
 
-    // Solve rhs = this * lhs.  Many blocks cannot implement this, so the default
-    // is to throw an error.
-    virtual void left_inverse(VectorView lhs,
-                              const ConstVectorView &rhs) const;
-    using SparseKalmanMatrix::left_inverse;
+    // Returns a solution to y = this * x.  This may raise an error if the
+    // solution is not well defined, which is the default.
+    virtual Vector left_inverse(const ConstVectorView &x) const;
   };
 
   //===========================================================================
@@ -217,6 +394,7 @@ namespace BOOM {
     void multiply_and_add(VectorView lhs,
                           const ConstVectorView &rhs) const override;
     void Tmult(VectorView lhs, const ConstVectorView &rhs) const override;
+
     void multiply_inplace(VectorView x) const override;
     void matrix_multiply_inplace(SubMatrix m) const override;
     void matrix_transpose_premultiply_inplace(SubMatrix m) const override;
@@ -262,6 +440,9 @@ namespace BOOM {
 
     Matrix dense() const override;
 
+    // Returns the "best least squares" solution.
+    Vector left_inverse(const ConstVectorView &x) const override;
+
    private:
     // Each block must have the same number of columns, which are determined by
     // the first block added.
@@ -290,6 +471,11 @@ namespace BOOM {
     SpdMatrix inner(const ConstVectorView &weights) const override;
     void add_to_block(SubMatrix block) const override;
     Matrix dense() const override;
+    Vector left_inverse(const ConstVectorView &x) const override {
+      Vector ans = x;
+      ans[0] -= ans[1];
+      return ans;
+    }
   };
 
   //======================================================================
@@ -326,6 +512,10 @@ namespace BOOM {
     // Fast access to the underlying matrix without returning a copy.
     const Matrix &matrix() const {return m_;}
 
+    Vector left_inverse(const ConstVectorView &x) const override {
+      return m_.solve(x);
+    }
+
    private:
     Matrix m_;
   };
@@ -362,6 +552,9 @@ namespace BOOM {
     SpdMatrix inner(const ConstVectorView &weights) const override {
       return m_.inner(weights);
     }
+    Vector left_inverse(const ConstVectorView &v) const override {
+      return m_.solve(v);
+    }
 
    private:
     SpdMatrix m_;
@@ -380,6 +573,10 @@ namespace BOOM {
       return value().inner(weights);
     }
 
+    Vector left_inverse(const ConstVectorView &v) const override {
+      return value().solve(v);
+    }
+
    private:
     Ptr<SpdParams> matrix_;
   };
@@ -444,6 +641,10 @@ namespace BOOM {
     void add_to_block(SubMatrix block) const override {
       block.diag() += diagonal_elements();
     }
+
+    Vector left_inverse(const ConstVectorView &x) const override {
+      return x / diagonal_elements();
+    }
   };
 
   //----------------------------------------------------------------------------
@@ -580,6 +781,7 @@ namespace BOOM {
     void add_to_block(SubMatrix block) const override;
     Matrix dense() const override;
 
+    Vector left_inverse(const ConstVectorView &x) const override;
    private:
     int number_of_seasons_;
   };
@@ -618,6 +820,7 @@ namespace BOOM {
     // virtual void matrix_transpose_premultiply_inplace(SubMatrix m) const;
     Matrix dense() const override;
 
+    Vector left_inverse(const ConstVectorView &x) const override;
    private:
     Ptr<GlmCoefs> autoregression_params_;
   };
@@ -648,10 +851,6 @@ namespace BOOM {
     void matrix_multiply_inplace(SubMatrix m) const override {}
     void matrix_transpose_premultiply_inplace(SubMatrix m) const override {}
     void add_to_block(SubMatrix block) const override { block.diag() += 1.0; }
-    void left_inverse(VectorView lhs,
-                      const ConstVectorView &rhs) const override {
-      lhs = rhs;
-    }
     SpdMatrix inner() const override {return SpdMatrix(nrow(), 1.0);}
     SpdMatrix inner(const ConstVectorView &weights) const override {
       SpdMatrix ans(nrow());
@@ -775,6 +974,62 @@ namespace BOOM {
     void add_to_block(SubMatrix block) const override {}
   };
 
+  // A 'matrix' with zero columns.  This is not really a 'Matrix' in the
+  // traditional sense.  Including a NullMatrix in a BlockDiagonalMatrix expands
+  // the number of rows (all 0's), but does not expand the number of columns.  A
+  // block diagonal matrix with 'regular' blocks B0 and B1 separated by a
+  // NullMatrix looks like:
+  //
+  // [B0 0]
+  // [0  0]
+  // [0 B1]
+  class NullMatrix: public SparseMatrixBlock {
+   public:
+    // Create a Nullmatrix with 'dim' rows and 0 columns.
+    explicit NullMatrix(int dim): dim_(dim) {}
+    NullMatrix *clone() const override {return new NullMatrix(*this);}
+    int ncol() const override {return 0;}
+    int nrow() const override {return dim_;}
+
+    void multiply(VectorView lhs, const ConstVectorView &rhs) const override {
+      conforms_to_rows(lhs.size());
+    }
+
+    void multiply_and_add(
+        VectorView lhs, const ConstVectorView &rhs) const override { }
+
+    void Tmult(VectorView lhs, const ConstVectorView &rhs) const override {
+      conforms_to_rows(rhs.size());
+      lhs = 0.0;
+    }
+
+    void multiply_inplace(VectorView x) const override {
+      report_error("Only square matrices can multiply_inplace.");
+    }
+    void matrix_multiply_inplace(SubMatrix x) const override {
+      report_error("Only square matrices can matrix_multiply_inplace.");
+    }
+    void matrix_transpose_premultiply_inplace(SubMatrix x) const override {
+      report_error("Only square matrices can matrix_transpose_"
+                   "premultiply_inplace.");
+    }
+
+    SpdMatrix inner() const override {
+      return SpdMatrix(nrow(), 0.0);
+    }
+
+    SpdMatrix inner(const ConstVectorView &weights) const override {
+      return this->inner();
+    }
+
+    void add_to_block(SubMatrix block) const override {
+      report_error("A NullMatrix cannot add_to_block.");
+    }
+
+   private:
+    int dim_;
+  };
+
   //======================================================================
   //  A matrix that is all zeros except for a single nonzero value in
   //  the (0,0) corner.
@@ -911,11 +1166,6 @@ namespace BOOM {
       return ans;
     }
 
-    void left_inverse(VectorView lhs,
-                      const ConstVectorView &rhs) const override {
-      lhs[0] = rhs[0];
-    }
-
    private:
     int nrow_;
   };
@@ -1313,11 +1563,18 @@ namespace BOOM {
   };
 
   //===========================================================================
-  // I - 11^T / dim.  This matrix removes the mean from a vector.  It is
-  // symmetric and idempotent.
+  // I - 11^T / dim.  This matrix removes the mean from a vector. In other
+  // words, if A is such a matrix, y is a vector, and ybar is the mean of the
+  // vector, then A * y = y - ybar.  It is symmetric and idempotent.
   class EffectConstraintMatrix : public SparseMatrixBlock {
    public:
-    explicit EffectConstraintMatrix(int dim) : dim_(dim) {}
+
+    // Args:
+    //   dim:  The notional number of rows and columns in the (square) matrix.
+    explicit EffectConstraintMatrix(int dim)
+        : dim_(dim)
+    {}
+
     EffectConstraintMatrix *clone() const override {
       return new EffectConstraintMatrix(*this);
     }
@@ -1366,6 +1623,110 @@ namespace BOOM {
     int dim_;
   };
 
+
+  // A matrix that subtracts off the mean of a regular subset of a vector.
+  class SubsetEffectConstraintMatrix : public SparseMatrixBlock {
+   public:
+
+    // Args:
+    //   dim:  The notional number of rows and columns in the (square) matrix.
+    //   stride:  The number of elements between the elements to be de-meaned.
+    //   offset:  The index of the first element to be de-meaned.
+    explicit SubsetEffectConstraintMatrix(int dim, int stride = 1, int offset = 0)
+        : dim_(dim),
+          stride_(stride),
+          offset_(offset)
+    {}
+
+    SubsetEffectConstraintMatrix *clone() const override {
+      return new SubsetEffectConstraintMatrix(*this);
+    }
+
+    int nrow() const override {return dim_;}
+
+    int ncol() const override {return dim_;}
+
+    void multiply(VectorView lhs, const ConstVectorView &rhs) const override {
+      lhs = rhs;
+      VectorView mean_subset = subset(lhs);
+      mean_subset -= mean(mean_subset);
+    }
+
+    void multiply_and_add(VectorView lhs, const ConstVectorView &rhs) const override {
+      Vector tmp(rhs.size(), 0.0);
+      VectorView tmp2(tmp);
+      multiply(tmp2, rhs);
+      lhs += tmp2;
+    }
+
+    void Tmult(VectorView lhs, const ConstVectorView &rhs) const override {
+      multiply(lhs, rhs);
+    }
+
+    void multiply_inplace(VectorView x) const override {
+      VectorView sub(subset(x));
+      sub -= mean(sub);
+    }
+
+    void add_to_block(SubMatrix block) const override {
+      conforms_to_rows(block.nrow());
+      conforms_to_cols(block.ncol());
+      block.diag() += 1.0;
+      int subset_dim = dim_ / stride_;
+      Vector decrement(dim_, 0.0);
+      subset(decrement) = 1.0 / subset_dim;
+      for (int j = offset_; j < dim_; j += stride_) {
+        block.col(j) -= decrement;
+      }
+    }
+
+    // Let J be the column vector of 0's and 1's picking off the subset to be
+    // demeaned.  Note that J'J = s.
+    //
+    // The inner product matrix is
+    // (I - JJ'/s)' * (I - JJ'/s)
+    // = I - JJ'/s - JJ'/s + JJ' * JJ' / s^2)
+    // = I + (-1/s -1/s + s/s^2)JJ'
+    // = I - JJ'/s
+    //
+    // So the matrix is idempotent.  This must be the matrix is symmetric, and
+    // because subtracting the mean is an idempotent operation.
+    SpdMatrix inner() const override {
+      return dense();
+    }
+
+    SpdMatrix inner(const ConstVectorView &weights) const override {
+      return dense().inner(weights);
+    }
+
+    Matrix dense() const override {
+      Matrix ans(dim_, dim_, 0.0);
+      ans.diag() += 1.0;
+      int subset_dim = dim_ / stride_;
+      Vector decrement(dim_, 0.0);
+      subset(decrement) = 1.0 / subset_dim;
+      for (int j = offset_; j < dim_; j += stride_) {
+        ans.col(j) -= decrement;
+      }
+      return ans;
+    }
+
+    VectorView subset(Vector &full_vector) const {
+      int subset_dim = dim_ / stride_;
+      return VectorView(full_vector.data() + offset_, subset_dim, stride_);
+    }
+
+    VectorView subset(VectorView &full_vector) const {
+      int subset_dim = dim_ / stride_;
+      return VectorView(full_vector.data() + offset_, subset_dim, stride_ * full_vector.stride());
+    }
+
+   private:
+    int dim_;
+    int stride_;
+    int offset_;
+  };
+
   //===========================================================================
   // The product of a SparseMatrixBlock and the matrix I - 11'/S, where I is the
   // identity matrix, 1 is a matrix of 1's, and everything is dimension S.  This
@@ -1448,6 +1809,185 @@ namespace BOOM {
     Ptr<SparseMatrixBlock> unconstrained_;
   };
 
+  //===========================================================================
+  // A SparseMatrixBlock formed by the product of two other SparseMatrixBlock's.
+  class ProductSparseMatrixBlock
+      : public SparseMatrixBlock {
+   public:
+    ProductSparseMatrixBlock(const Ptr<SparseMatrixBlock> left,
+                             const Ptr<SparseMatrixBlock> &right)
+        : left_(left),
+          right_(right)
+    {}
+    ProductSparseMatrixBlock(const ProductSparseMatrixBlock &rhs)
+        : SparseMatrixBlock(rhs),
+          left_(rhs.left_->clone()),
+          right_(rhs.right_->clone())
+    {}
+    ProductSparseMatrixBlock & operator=(const ProductSparseMatrixBlock &rhs) {
+      if (&rhs != this) {
+        SparseMatrixBlock::operator=(rhs);
+        left_ = rhs.left_->clone();
+        right_ = rhs.right_->clone();
+      }
+      return *this;
+    }
+
+    ProductSparseMatrixBlock(ProductSparseMatrixBlock &&rhs) = default;
+    ProductSparseMatrixBlock &operator=(ProductSparseMatrixBlock &&rhs) = default;
+
+    ProductSparseMatrixBlock * clone() const override {
+      return new ProductSparseMatrixBlock(*this);
+    }
+
+    int nrow() const override {return left_->nrow();}
+
+    int ncol() const override {return right_->ncol();}
+
+    void multiply(VectorView lhs, const ConstVectorView &rhs) const override {
+      Vector tmp(right_->nrow());
+      right_->multiply(VectorView(tmp), rhs);
+      left_->multiply(lhs, tmp);
+    }
+
+    void multiply_and_add(VectorView lhs,
+                          const ConstVectorView &rhs) const override {
+      Vector tmp(right_->nrow());
+      right_->multiply(VectorView(tmp), rhs);
+      left_->multiply_and_add(lhs, tmp);
+    }
+
+    void Tmult(VectorView lhs, const ConstVectorView &rhs) const override {
+      Vector tmp(left_->ncol());
+      left_->Tmult(VectorView(tmp), rhs);
+      right_->Tmult(lhs, tmp);
+    }
+
+    void multiply_inplace(VectorView x) const override {
+      right_->multiply_inplace(x);
+      left_->multiply_inplace(x);
+    }
+
+    SpdMatrix inner() const override {
+      return left_->sandwich_transpose(right_->inner());
+    }
+
+    SpdMatrix inner(const ConstVectorView &weights) const override {
+      return left_->sandwich_transpose(right_->inner(weights));
+    }
+
+    void add_to_block(SubMatrix block) const override {
+    }
+
+    Matrix dense() const override {
+      return (*left_) * right_->dense();
+    }
+
+    Vector left_inverse(const ConstVectorView &x) const override {
+      return right_->left_inverse(left_->left_inverse(x));
+    }
+
+   private:
+    Ptr<SparseMatrixBlock> left_;
+    Ptr<SparseMatrixBlock> right_;
+  };
+
+  //===========================================================================
+  // The product d * s of a dense "column vector" d and a sparse "row vector" s.
+  class DenseSparseRankOneMatrixBlock : public SparseMatrixBlock {
+   public:
+    DenseSparseRankOneMatrixBlock(const Vector &left, const SparseVector &right)
+        : left_(left),
+          right_(right),
+          dense_right_(0)
+    {}
+
+    DenseSparseRankOneMatrixBlock * clone() const override {
+      return new DenseSparseRankOneMatrixBlock(*this);
+    }
+
+    void set_left(const Vector &left) {
+      left_ = left;
+    }
+
+    void set_right(const SparseVector &right) {
+      right_ = right;
+      dense_right_ = Vector(0);
+    }
+
+    void update(const Vector &left, const SparseVector &right) {
+      set_left(left);
+      set_right(right);
+    }
+
+    int nrow() const override {return left_.size();}
+    int ncol() const override {return right_.size();}
+    void multiply(VectorView lhs, const ConstVectorView &rhs) const override {
+      lhs = left_ * right_.dot(rhs);
+    }
+
+    void multiply_and_add(VectorView lhs, const ConstVectorView &rhs) const override {
+      lhs += left_ * right_.dot(rhs);
+    }
+
+    void Tmult(VectorView lhs, const ConstVectorView &rhs) const override {
+      lhs = left_.dot(rhs) * dense_right();
+    }
+
+    void multiply_inplace(VectorView x) const override {
+      if (nrow() != ncol()) {
+        report_error("multiply_inplace only works for square matrices.");
+      }
+      if (ncol() != x.size()) {
+        report_error("Vector does not conform to matrix in multiply_inplace.");
+      }
+      x = left_ * right_.dot(x);
+    }
+
+    SpdMatrix inner() const override {
+      double weight = left_.dot(left_);
+      return outer(dense_right() * sqrt(weight));
+    }
+
+    SpdMatrix inner(const ConstVectorView &weights) const override {
+      //  R' L' W L R
+      double weight = left_.dot(weights * left_);
+      return outer(dense_right() * sqrt(weight));
+    }
+
+    Matrix &add_to(Matrix &P) const override {
+      P += right_.outer_product_transpose(left_);
+      return P;
+    }
+
+    SubMatrix add_to_submatrix(SubMatrix P) const override {
+      P += right_.outer_product_transpose(left_);
+      return P;
+    }
+
+    void add_to_block(SubMatrix block) const override{
+      add_to_submatrix(block);
+    }
+
+    Matrix dense() const override {
+      Matrix ans(nrow(), ncol(), 0.0);
+      ans.add_outer(left_, dense_right());
+      return ans;
+    }
+
+    const Vector & dense_right() const {
+      if (dense_right_.size() != right_.size()) {
+        dense_right_ = right_.dense();
+      }
+      return dense_right_;
+    }
+
+   private:
+    Vector left_;
+    SparseVector right_;
+    mutable Vector dense_right_;
+  };
+
   //===========================================================================
   // A sparse matrix whose rows and columns are sparse vectors.  This matrix is
   // somewhat expensive to construct, because a mapping must be constructed
@@ -1570,11 +2110,18 @@ namespace BOOM {
   // The state transition equation for a dynamic linear model will
   // typically involve a block diagonal matrix.  The blocks will
   // typically be:  SeasonalStateSpaceMatrix, IdentityMatrix, etc.
-  class BlockDiagonalMatrix : public SparseKalmanMatrix {
+  class BlockDiagonalMatrix : public SparseMatrixBlock {
    public:
     // Start off with an empty matrix.  Use add_block() to add blocks
     // Adds a block to the block diagonal matrix
     BlockDiagonalMatrix();
+    BlockDiagonalMatrix(const BlockDiagonalMatrix &rhs);
+    BlockDiagonalMatrix & operator=(const BlockDiagonalMatrix &rhs);
+
+    BlockDiagonalMatrix(BlockDiagonalMatrix &&rhs) = default;
+    BlockDiagonalMatrix & operator=(BlockDiagonalMatrix &&rhs) = default;
+
+    BlockDiagonalMatrix * clone() const override;
 
     void add_block(const Ptr<SparseMatrixBlock> &m);
     void replace_block(int which_block, const Ptr<SparseMatrixBlock> &b);
@@ -1583,6 +2130,15 @@ namespace BOOM {
     int nrow() const override;
     int ncol() const override;
 
+    void multiply(VectorView lhs,
+                  const ConstVectorView &rhs) const override;
+    void multiply_and_add(VectorView lhs,
+                          const ConstVectorView &rhs) const override;
+    void multiply_inplace(VectorView x) const override;
+    void add_to_block(SubMatrix block) const override {
+      add_to_submatrix(block);
+    }
+
     Vector operator*(const Vector &v) const override;
     Vector operator*(const VectorView &v) const override;
     Vector operator*(const ConstVectorView &v) const override;
@@ -1592,6 +2148,7 @@ namespace BOOM {
     using SparseKalmanMatrix::operator*;
 
     using SparseKalmanMatrix::Tmult;
+    void Tmult(VectorView lhs, const ConstVectorView &x) const override;
     Vector Tmult(const ConstVectorView &x) const override;
     SpdMatrix inner() const override;
     SpdMatrix inner(const ConstVectorView &weights) const override;
@@ -1606,31 +2163,12 @@ namespace BOOM {
     Matrix &add_to(Matrix &P) const override;
     SubMatrix add_to_submatrix(SubMatrix P) const override;
 
-    // Solve the equation rhs = this * lhs.  It may be the case that some matrix
-    // blocks are non-invertible, in which case calling this function will
-    // report an error.
-    //
-    // The initial reason for this implementing this method is to evaluate the
-    // transition density for state space models when the error distribution is
-    // less than full rank.   In those settings we must be able to evaluate
-    // error_expander->left_inverse( new_state - T * old_state).
-    //
-    // Thus the only blocks that need to implement left_inverse are those used
-    // in an error expander somewhere.
-    Vector left_inverse(const ConstVectorView &rhs) const override;
-
    private:
     // Replace middle with left * middle * right.transpose()
     void sandwich_inplace_block(const SparseMatrixBlock &left,
                                 const SparseMatrixBlock &right,
                                 SubMatrix middle) const;
 
-    // Fills dest with left * source * right.transpose().
-    void sandwich_block(const SparseMatrixBlock &left,
-                        const SparseMatrixBlock &right,
-                        const ConstSubMatrix &source, SubMatrix &dest,
-                        Matrix &workspace) const;
-
     // Returns the (i,j) block of the matrix m, with block sizes
     // determined by the rows and columns of the entries in blocks_.
     SubMatrix get_block(Matrix &m, int i, int j) const;
@@ -1686,7 +2224,92 @@ namespace BOOM {
     std::vector<Ptr<SparseMatrixBlock>> blocks_;
   };
 
-  //======================================================================
+  //===========================================================================
+  // A ErrorExpanderMatrix is a BlockDiagonalMatrix that allows for blocks that
+  // have ncol==0.  The effect of such blocks is to add block->nrow() rows of
+  // 0's to the dense representation of the matrix.
+  class ErrorExpanderMatrix : public SparseMatrixBlock {
+   public:
+    ErrorExpanderMatrix();
+
+    ErrorExpanderMatrix(const ErrorExpanderMatrix &rhs);
+    ErrorExpanderMatrix & operator=(const ErrorExpanderMatrix &rhs);
+
+    ErrorExpanderMatrix(ErrorExpanderMatrix &&rhs) = default;
+    ErrorExpanderMatrix & operator=(ErrorExpanderMatrix &&rhs) = default;
+
+    ErrorExpanderMatrix * clone() const override;
+
+    // Seasonal state models may change their number of columns over time,
+    // because there will be one additional element of "noise" in time periods
+    // when a new season begins.  For this reason we don't assume nrow and ncol
+    // are fixed.  They get recomputed each time.
+    int nrow() const override;
+    int ncol() const override;
+
+    void add_block(const Ptr<SparseMatrixBlock> &block);
+    //     void add_block(const Ptr<ErrorExpanderMatrix> &blocks);
+    void replace_block(int block_index,
+                       const Ptr<SparseMatrixBlock> &block);
+
+    // Remove all the blocks, making the matrix empty.
+    void clear();
+
+    void multiply(VectorView lhs, const ConstVectorView &rhs) const override;
+    void multiply_and_add(VectorView lhs, const ConstVectorView &rhs) const override;
+    void add_to_block(SubMatrix block) const override;
+
+    // Will throw an exception if any blocks are non-square.
+    void multiply_inplace(VectorView x) const override;
+
+    Vector operator*(const Vector &v) const override;
+    Vector operator*(const VectorView &v) const override;
+    Vector operator*(const ConstVectorView &v) const override;
+
+    // Use the base class matrix-matrix multiply operations.
+    using SparseKalmanMatrix::operator*;
+    using SparseKalmanMatrix::Tmult;
+
+    Vector Tmult(const ConstVectorView &x) const override;
+    void Tmult(VectorView lhs, const ConstVectorView &rhs) const override;
+
+    SpdMatrix inner() const override;
+    SpdMatrix inner(const ConstVectorView &weights) const override;
+
+    // These operations will raise errors if the ErrorExpanderMatrix is not
+    // square.
+    void sandwich_inplace(SpdMatrix &P) const override;
+    void sandwich_inplace_submatrix(SubMatrix P) const override;
+
+    // sandwich(P) = this * P * this.transpose()
+    SpdMatrix sandwich(const SpdMatrix &P) const override;
+
+    Matrix &add_to(Matrix &P) const override;
+    SubMatrix add_to_submatrix(SubMatrix P) const override;
+
+    // Solve the equation rhs = this * lhs.  It may be the case that some matrix
+    // blocks are non-invertible, in which case calling this function will
+    // report an error.
+    //
+    // The initial reason for this implementing this method is to evaluate the
+    // transition density for state space models when the error distribution is
+    // less than full rank.   In those settings we must be able to evaluate
+    // error_expander->left_inverse( new_state - T * old_state).
+    Vector left_inverse(const ConstVectorView &rhs) const override;
+
+   private:
+    std::vector<Ptr<SparseMatrixBlock>> blocks_;
+
+    int nrow_;
+    int ncol_;
+    std::vector<int> row_boundaries_;
+    std::vector<int> col_boundaries_;
+
+    void recompute_sizes();
+    void increment_sizes(const Ptr<SparseMatrixBlock> &block);
+  };
+
+  //===========================================================================
 
   Vector operator*(const SparseMatrixBlock &, const Vector &);
 
