@@ -40,6 +40,13 @@ namespace BOOM {
                1.0);
   }
 
+  DPS *DPS::clone_to_new_host(Model *new_host) const {
+    return new DPS(dynamic_cast<DirichletModel *>(new_host),
+                   phi_prior_->clone(),
+                   alpha_prior_->clone(),
+                   rng());
+  }
+
   double DPS::logpri() const {
     const Vector &nu(model_->nu());
     double alpha = sum(nu);
@@ -468,7 +475,7 @@ namespace BOOM {
         jacobian.add_eta_gradient(gradient, jacobian_matrix);
 
         if (nderiv > 1) {
-          Hessian = sandwich(jacobian_matrix.t(), phi_hessian);
+          Hessian = sandwich(jacobian_matrix.transpose(), phi_hessian);
           for (int r = 0; r < phi_gradient.size(); ++r) {
             for (int s = 0; s < phi_gradient.size(); ++s) {
               for (int t = 0; t < phi_gradient.size(); ++t) {
