@@ -1,3 +1,4 @@
+// Copyright 2018 Google LLC. All Rights Reserved.
 /*
   Copyright (C) 2005-2009 Steven L. Scott
 
@@ -16,32 +17,30 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
 
-#include <Models/PosteriorSamplers/SharedSigsqSampler.hpp>
-#include <distributions.hpp>
+#include "Models/PosteriorSamplers/SharedSigsqSampler.hpp"
+#include "distributions.hpp"
 
-namespace BOOM{
+namespace BOOM {
 
   typedef SharedSigsqSampler SSS;
 
-  SSS::SharedSigsqSampler(const std::vector<GaussianModelBase*> &models,
-                          Ptr<UnivParams> sigsq,
-                          Ptr<GammaModelBase> pri,
-                          RNG &seeding_rng)
-    : PosteriorSampler(seeding_rng),
-      models_(models),
-      sigsq_(sigsq),
-      pri_(pri),
-      sigsq_sampler_(pri_)
-  {}
+  SSS::SharedSigsqSampler(const std::vector<GaussianModelBase *> &models,
+                          const Ptr<UnivParams> &sigsq,
+                          const Ptr<GammaModelBase> &pri, RNG &seeding_rng)
+      : PosteriorSampler(seeding_rng),
+        models_(models),
+        sigsq_(sigsq),
+        pri_(pri),
+        sigsq_sampler_(pri_) {}
 
-  void SSS::draw(){
+  void SSS::draw() {
     double df = 0;
     double ss = 0;
 
-    for(uint i=0; i<models_.size(); ++i){
+    for (uint i = 0; i < models_.size(); ++i) {
       Ptr<GaussianSuf> suf = models_[i]->suf();
       double n = suf->n();
-      df+= n;
+      df += n;
       double mu = models_[i]->mu();
       ss += suf->sumsq() + mu * suf->sum() + n * mu * mu;
     }
@@ -50,8 +49,8 @@ namespace BOOM{
     sigsq_->set(sigsq);
   }
 
-  double SSS::logpri()const{
+  double SSS::logpri() const {
     return sigsq_sampler_.log_prior(sigsq_->value());
   }
 
-}
+}  // namespace BOOM
